!--------------------------------------------------------------------
subroutine setphs(ph,ps,is1,ie1,js1,je1,ks1,ke1)
!--------------------------------------------------------------------
!         Addititonal variable Phi & Phi for GLM-MHD
!         Initial Setup for Phi & Phi
!
  use pram, only : imax, jmax, kmax
  implicit none

  integer :: i, j, k, is1, ie1, js1, je1, ks1, ke1

  real(8) :: ph(is1:ie1,js1:je1,ks1:ke1), &
             ps(is1:ie1,js1:je1,ks1:ke1)
!
!--------------------------------------------------------------------
!
  do k=ks1,ke1
    do j=js1,je1
      do i=is1,ie1

         ph(i,j,k)=0.d0
         ps(i,j,k)=0.d0

      enddo
    enddo
  enddo
 
  return
end subroutine setphs
!
!--------------------------------------------------------------------
subroutine calglmch1(glmch,glmcp,dt,dx1a,dx2a,dx3a)
!--------------------------------------------------------------------
!         Calculation of C_h for GLM-MHD 
!
  use pram, only : cfl, glmcr
  implicit none

  real(8) :: glmch, glmcp, dt, dx1a, dx2a, dx3a
!
!--------------------------------------------------------------------
!
!  glmch=cfl*min(dx1a,dx2a,dx3a)/dt
  glmch=1.d0
  glmcp=1.d0
!  glmcp=sqrt(glmch*glmcr)
 
  return
end subroutine calglmch1
!
!---------------------------------------------------------------------@
subroutine glmhll(ww,uuir,uujr,uukr,uuil,uujl,uukl, &
                wwir,wwjr,wwkr,wwil,wwjl,wwkl, &
                glmch,nm0,is1,ie1,js1,je1,ks1,ke1)
!---------------------------------------------------------------------@
!
!     Calculate numerical flux of GLM part (Psi & Phi) by HLLE method
!
  use pram, only : imax, jmax, kmax, nv
  implicit none

  integer :: i, j, k
  integer :: nm0, is1, ie1, js1, je1, ks1, ke1

  real(8) :: uuir(nv,is1:ie1,js1:je1,ks1:ke1), &
             uuil(nv,is1:ie1,js1:je1,ks1:ke1), & 
             uujr(nv,is1:ie1,js1:je1,ks1:ke1), &
             uujl(nv,is1:ie1,js1:je1,ks1:ke1), &
             uukr(nv,is1:ie1,js1:je1,ks1:ke1), &
             uukl(nv,is1:ie1,js1:je1,ks1:ke1)
     
  real(8) :: ww(3,nv,is1:ie1,js1:je1,ks1:ke1), &
             wwir(nv,is1:ie1,js1:je1,ks1:ke1), &
             wwil(nv,is1:ie1,js1:je1,ks1:ke1), &
             wwjr(nv,is1:ie1,js1:je1,ks1:ke1), &
             wwjl(nv,is1:ie1,js1:je1,ks1:ke1), &
             wwkr(nv,is1:ie1,js1:je1,ks1:ke1), &
             wwkl(nv,is1:ie1,js1:je1,ks1:ke1)

  real(8) :: glmch
!
!=====================================================================@

  do k=ks1+nm0-1,ke1-nm0
    do j=js1+nm0-1,je1-nm0
      do i=is1+nm0-1,ie1-nm0
! for phi
!         ww(1,14,i,j,k)=0.5*( wwil(14,i,j,k)+wwir(14,i,j,k) - &
!                             (uuir(14,i,j,k)-uuil(14,i,j,k)) )
!         ww(2,14,i,j,k)=0.5*( wwjl(14,i,j,k)+wwjr(14,i,j,k) - &
!                             (uujr(14,i,j,k)-uujl(14,i,j,k)) )
!         ww(3,14,i,j,k)=0.5*( wwkl(14,i,j,k)+wwkr(14,i,j,k) - &
!                             (uukr(14,i,j,k)-uukl(14,i,j,k)) )
 
         ww(1,14,i,j,k)=(glmch*wwil(14,i,j,k) &
                        +glmch*wwir(14,i,j,k) &
                        -glmch*glmch*(uuir(14,i,j,k)-uuil(14,i,j,k))) &
                        /(glmch+glmch) 
!
         ww(2,14,i,j,k)=(glmch*wwjl(14,i,j,k) &
                        +glmch*wwjr(14,i,j,k) &
                        -glmch*glmch*(uujr(14,i,j,k)-uujl(14,i,j,k))) &
                        /(glmch+glmch) 
!
         ww(3,14,i,j,k)=(glmch*wwkl(14,i,j,k) &
                        +glmch*wwkr(14,i,j,k) &
                        -glmch*glmch*(uukr(14,i,j,k)-uukl(14,i,j,k))) &
                        /(glmch+glmch) 
! for psi
!         ww(1,15,i,j,k)=0.5*( wwil(15,i,j,k)+wwir(15,i,j,k) - &
!                             (uuir(15,i,j,k)-uuil(15,i,j,k)) )
!         ww(2,15,i,j,k)=0.5*( wwjl(15,i,j,k)+wwjr(15,i,j,k) - &
!                             (uujr(15,i,j,k)-uujl(15,i,j,k)) )
!         ww(3,15,i,j,k)=0.5*( wwkl(15,i,j,k)+wwkr(15,i,j,k) - &
!                             (uukr(15,i,j,k)-uukl(15,i,j,k)) )


         ww(1,15,i,j,k)=(glmch*wwil(15,i,j,k) &
                        +glmch*wwir(15,i,j,k) &
                        -glmch*glmch*(uuir(15,i,j,k)-uuil(15,i,j,k))) &
                        /(glmch+glmch) 
!
         ww(2,15,i,j,k)=(glmch*wwjl(15,i,j,k) &
                        +glmch*wwjr(15,i,j,k) &
                        -glmch*glmch*(uujr(15,i,j,k)-uujl(15,i,j,k))) &
                        /(glmch+glmch) 
!
         ww(3,15,i,j,k)=(glmch*wwkl(15,i,j,k) &
                        +glmch*wwkr(15,i,j,k) &
                        -glmch*glmch*(uukr(15,i,j,k)-uukl(15,i,j,k))) &
                        /(glmch+glmch) 
      enddo
    enddo
  enddo
   
  return
end subroutine glmhll
!
!---------------------------------------------------------------------@
subroutine glmrecov(uu,uri,nm0,is1,ie1,js1,je1,ks1,ke1)
!---------------------------------------------------------------------@
!
!     Calculate primitive variables in GLM terms
!
  use pram, only : imax, jmax, kmax, nv, ns
  implicit none

  integer :: i, j, k
  integer :: nm0, is1, ie1, js1, je1, ks1, ke1

  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1), &
             uri(ns,is1:ie1,js1:je1,ks1:ke1)
!
!=====================================================================@

  do k=ks1+nm0,ke1-nm0
    do j=js1+nm0,je1-nm0
      do i=is1+nm0,ie1-nm0

        uri(10,i,j,k)=uu(10,i,j,k) ! E_1
        uri(11,i,j,k)=uu(11,i,j,k) ! E_2
        uri(12,i,j,k)=uu(12,i,j,k) ! E_3         
        uri(14,i,j,k)=uu(14,i,j,k) ! Phi
        uri(15,i,j,k)=uu(15,i,j,k) ! Psi

      enddo
    enddo
  enddo
   
  return
end subroutine glmrecov
!---------------------------------------------------------------------@
subroutine glmrecov1(uu,uri,is1,ie1,js1,je1,ks1,ke1)
!---------------------------------------------------------------------@
!
!     Calculate primitive variables in GLM terms
!
  use pram, only : imax, jmax, kmax, nv, ns
  implicit none

  integer :: i, j, k
  integer :: is1, ie1, js1, je1, ks1, ke1

  real(8) :: uu(nv,is1:ie1,js1:je1,ks1:ke1), &
             uri(ns,is1:ie1,js1:je1,ks1:ke1)
!
!=====================================================================@

  do k=ks1,ke1
    do j=js1,je1
      do i=is1,ie1
          
        uri(10,i,j,k)=uu(10,i,j,k) ! E_1
        uri(11,i,j,k)=uu(11,i,j,k) ! E_2
        uri(12,i,j,k)=uu(12,i,j,k) ! E_3
        uri(14,i,j,k)=uu(14,i,j,k) ! Phi
        uri(15,i,j,k)=uu(15,i,j,k) ! Psi

      enddo
    enddo
  enddo
   
  return
end subroutine glmrecov1
